package cn.ibizlab.businesscentral.core.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.businesscentral.core.dto.*;
import cn.ibizlab.businesscentral.core.mapping.*;
import cn.ibizlab.businesscentral.core.odoo_purchase.domain.Purchase_requisition_line;
import cn.ibizlab.businesscentral.core.odoo_purchase.service.IPurchase_requisition_lineService;
import cn.ibizlab.businesscentral.core.odoo_purchase.filter.Purchase_requisition_lineSearchContext;
import cn.ibizlab.businesscentral.util.annotation.VersionCheck;

@Slf4j
@Api(tags = {"采购申请行" })
@RestController("Core-purchase_requisition_line")
@RequestMapping("")
public class Purchase_requisition_lineResource {

    @Autowired
    public IPurchase_requisition_lineService purchase_requisition_lineService;

    @Autowired
    @Lazy
    public Purchase_requisition_lineMapping purchase_requisition_lineMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Create-all')")
    @ApiOperation(value = "新建采购申请行", tags = {"采购申请行" },  notes = "新建采购申请行")
	@RequestMapping(method = RequestMethod.POST, value = "/purchase_requisition_lines")
    public ResponseEntity<Purchase_requisition_lineDTO> create(@Validated @RequestBody Purchase_requisition_lineDTO purchase_requisition_linedto) {
        Purchase_requisition_line domain = purchase_requisition_lineMapping.toDomain(purchase_requisition_linedto);
		purchase_requisition_lineService.create(domain);
        Purchase_requisition_lineDTO dto = purchase_requisition_lineMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Create-all')")
    @ApiOperation(value = "批量新建采购申请行", tags = {"采购申请行" },  notes = "批量新建采购申请行")
	@RequestMapping(method = RequestMethod.POST, value = "/purchase_requisition_lines/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<Purchase_requisition_lineDTO> purchase_requisition_linedtos) {
        purchase_requisition_lineService.createBatch(purchase_requisition_lineMapping.toDomain(purchase_requisition_linedtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Update-all')")
    @ApiOperation(value = "更新采购申请行", tags = {"采购申请行" },  notes = "更新采购申请行")
	@RequestMapping(method = RequestMethod.PUT, value = "/purchase_requisition_lines/{purchase_requisition_line_id}")
    public ResponseEntity<Purchase_requisition_lineDTO> update(@PathVariable("purchase_requisition_line_id") Long purchase_requisition_line_id, @RequestBody Purchase_requisition_lineDTO purchase_requisition_linedto) {
		Purchase_requisition_line domain  = purchase_requisition_lineMapping.toDomain(purchase_requisition_linedto);
        domain .setId(purchase_requisition_line_id);
		purchase_requisition_lineService.update(domain );
		Purchase_requisition_lineDTO dto = purchase_requisition_lineMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Update-all')")
    @ApiOperation(value = "批量更新采购申请行", tags = {"采购申请行" },  notes = "批量更新采购申请行")
	@RequestMapping(method = RequestMethod.PUT, value = "/purchase_requisition_lines/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<Purchase_requisition_lineDTO> purchase_requisition_linedtos) {
        purchase_requisition_lineService.updateBatch(purchase_requisition_lineMapping.toDomain(purchase_requisition_linedtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Remove-all')")
    @ApiOperation(value = "删除采购申请行", tags = {"采购申请行" },  notes = "删除采购申请行")
	@RequestMapping(method = RequestMethod.DELETE, value = "/purchase_requisition_lines/{purchase_requisition_line_id}")
    public ResponseEntity<Boolean> remove(@PathVariable("purchase_requisition_line_id") Long purchase_requisition_line_id) {
         return ResponseEntity.status(HttpStatus.OK).body(purchase_requisition_lineService.remove(purchase_requisition_line_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Remove-all')")
    @ApiOperation(value = "批量删除采购申请行", tags = {"采购申请行" },  notes = "批量删除采购申请行")
	@RequestMapping(method = RequestMethod.DELETE, value = "/purchase_requisition_lines/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<Long> ids) {
        purchase_requisition_lineService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Get-all')")
    @ApiOperation(value = "获取采购申请行", tags = {"采购申请行" },  notes = "获取采购申请行")
	@RequestMapping(method = RequestMethod.GET, value = "/purchase_requisition_lines/{purchase_requisition_line_id}")
    public ResponseEntity<Purchase_requisition_lineDTO> get(@PathVariable("purchase_requisition_line_id") Long purchase_requisition_line_id) {
        Purchase_requisition_line domain = purchase_requisition_lineService.get(purchase_requisition_line_id);
        Purchase_requisition_lineDTO dto = purchase_requisition_lineMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "获取采购申请行草稿", tags = {"采购申请行" },  notes = "获取采购申请行草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/purchase_requisition_lines/getdraft")
    public ResponseEntity<Purchase_requisition_lineDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(purchase_requisition_lineMapping.toDto(purchase_requisition_lineService.getDraft(new Purchase_requisition_line())));
    }

    @ApiOperation(value = "检查采购申请行", tags = {"采购申请行" },  notes = "检查采购申请行")
	@RequestMapping(method = RequestMethod.POST, value = "/purchase_requisition_lines/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody Purchase_requisition_lineDTO purchase_requisition_linedto) {
        return  ResponseEntity.status(HttpStatus.OK).body(purchase_requisition_lineService.checkKey(purchase_requisition_lineMapping.toDomain(purchase_requisition_linedto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Save-all')")
    @ApiOperation(value = "保存采购申请行", tags = {"采购申请行" },  notes = "保存采购申请行")
	@RequestMapping(method = RequestMethod.POST, value = "/purchase_requisition_lines/save")
    public ResponseEntity<Boolean> save(@RequestBody Purchase_requisition_lineDTO purchase_requisition_linedto) {
        return ResponseEntity.status(HttpStatus.OK).body(purchase_requisition_lineService.save(purchase_requisition_lineMapping.toDomain(purchase_requisition_linedto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-Save-all')")
    @ApiOperation(value = "批量保存采购申请行", tags = {"采购申请行" },  notes = "批量保存采购申请行")
	@RequestMapping(method = RequestMethod.POST, value = "/purchase_requisition_lines/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<Purchase_requisition_lineDTO> purchase_requisition_linedtos) {
        purchase_requisition_lineService.saveBatch(purchase_requisition_lineMapping.toDomain(purchase_requisition_linedtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-searchDefault-all')")
	@ApiOperation(value = "获取数据集", tags = {"采购申请行" } ,notes = "获取数据集")
    @RequestMapping(method= RequestMethod.GET , value="/purchase_requisition_lines/fetchdefault")
	public ResponseEntity<List<Purchase_requisition_lineDTO>> fetchDefault(Purchase_requisition_lineSearchContext context) {
        Page<Purchase_requisition_line> domains = purchase_requisition_lineService.searchDefault(context) ;
        List<Purchase_requisition_lineDTO> list = purchase_requisition_lineMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','iBizBusinessCentral-Purchase_requisition_line-searchDefault-all')")
	@ApiOperation(value = "查询数据集", tags = {"采购申请行" } ,notes = "查询数据集")
    @RequestMapping(method= RequestMethod.POST , value="/purchase_requisition_lines/searchdefault")
	public ResponseEntity<Page<Purchase_requisition_lineDTO>> searchDefault(@RequestBody Purchase_requisition_lineSearchContext context) {
        Page<Purchase_requisition_line> domains = purchase_requisition_lineService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(purchase_requisition_lineMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}


}

